# Import third-party modules
import pytest

# Import local modules
from arthub_login_widgets.core import LoginWindow
from arthub_login_widgets.core import LoginBackend
from arthub_login_widgets.test.ipc import IPCServer


class FakeAPI:
    def __init__(self, mocker, login_status=True):
        self.login_status = login_status
        self.mocker = mocker

    def login(self, account, password, save_token_to_cache):
        return self.mocker.MagicMock(
            is_succeeded=self.mocker.MagicMock(return_value=self.login_status),
            error_message=self.mocker.MagicMock(return_value="login failed."),
        )


class TestLoginWindow:
    @pytest.fixture(autouse=True)
    def setup(self, mocker, qtbot):
        self.widget = LoginWindow(api=FakeAPI(mocker))
        qtbot.addWidget(self.widget)

    def test_show_widget(self):
        self.widget.line_edit_password.setText("xxxx")
        assert self.widget.line_edit_password.text() == "xxxx"

    def test_run_callback(self):
        def _custom_callback(api):
            print("run custom callback...")
            api._custom_callback = "custom callback"
            assert True

        self.widget.line_edit_account.setText("abc")
        self.widget.line_edit_password.setText("abc")
        self.widget.set_callback(_custom_callback)
        self.widget.login()
        assert self.widget._custom_callback == "custom callback"

    def test_login_failed(self, mocker):
        widget = LoginWindow(FakeAPI(mocker=mocker, login_status=False))
        widget.line_edit_account.setText("abc")
        widget.line_edit_password.setText("abc")

        widget.login()
        assert widget.label_prompt.text() == "login failed."


def test_task_pad():
    # init account backend
    backend = LoginBackend(
        terminal_type="dcc",
        business_type="default",
        dev_mode=True,
    )
    if not backend.is_login():
        assert backend.popup_login()

    def process_client_request(m):
        print("receive message from client! {}".format(m))
        # Will not response if return None
        return None

    # init ipc server
    server = IPCServer()
    # set request process function
    server.set_receive_callback(process_client_request)
    print("start server..")
    # listen at port {0} will find a usable port
    port = server.init(0)
    print("listen at {}".format(port))

    # show task pad window
    window = backend.popup_task_pad(port)

    while True:
        text = input("input command: ")
        if text == "e":
            break

        message = {
            "command": text,
            "filepath": "G:/MotionBuillderFiles/A_S2_Idle_to_RunL180_PI_o_001_walk_stand_none_two_layer.fbx"
        }
        server.send_message(json.dumps(message).encode('utf-8'))

    print("server closing..")
    server.close_server()

    print("window closing..")
    window.close()



"""Describe our module distribution to Distutils."""

# Import future modules
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

# Import third-party modules
from setuptools import find_packages
from setuptools import setup
import os
from codecs import open

about = {}
here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, "arthub_login_widgets", "__version__.py"), "r", "utf-8") as f:
    exec(f.read(), about)

resource_files = []
for root, _, files in os.walk(os.path.join(here, 'arthub_login_widgets', 'resources', 'client')):
    for file in files:
        resource_files.append(os.path.relpath(os.path.join(root, file), 'arthub_login_widgets'))

setup(
    name="arthub_login_widgets",
    version=about["__version__"],
    author="Joey Ding",
    author_email="joeyding@tencent.com",
    url="https://git.woa.com/lightbox/internal/arthub_login_widgets",
    package_dir={"": "."},
    packages=find_packages("."),
    description="A Qt Widget for login ArtHub.",
    entry_points={},
    include_package_data=True,
    package_data={"arthub_login_widgets": ["*.png", "*.qss"] + resource_files},
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
    ],
    install_requires=[
        "arthub_api>=1.8.0",
        "platformdirs==2.0.2",
        "Qt.py"
    ]
)
